@@ -187,6 +187,7 @@ func TestParse(t *testing.T) {
 		{"async function a(){ for await (var a of b) {} }", "Decl(async function a Params() Stmt({ Stmt(for await Decl(var Binding(a)) of b Stmt({ })) }))"},
 		{"async function a(){ (await a) }", "Decl(async function a Params() Stmt({ Stmt((await a)) }))"},
 		{"x = {async a(b){}}", "Stmt(x={Method(async a Params(Binding(b)) Stmt({ }))})"},
+		{"async(...a)", "Stmt(async(...a))"},
 
 		// bindings
 		{"let [] = z", "Decl(let Binding([ ] = z))"},
@@ -402,8 +403,10 @@ func TestParse(t *testing.T) {
 		{"function f(){return /*comment*/ a}", "Decl(function f Params() Stmt({ Stmt(return a) }))"},
 		{"function f(){return /*com\nment*/ a}", "Decl(function f Params() Stmt({ Stmt(return) Stmt(a) }))"},
 		{"function f(){return //comment\n a}", "Decl(function f Params() Stmt({ Stmt(return) Stmt(a) }))"},
-		{"a?.b\n`c`", "Stmt((a?.b)`c`)"},
+		{"if(a)while(true)\n;else;", "Stmt(if a Stmt(while true Stmt()) else Stmt())"},
+		{"if(a)for(;;)\n;else;", "Stmt(if a Stmt(for ; ; Stmt({ })) else Stmt())"},
 
+		{"a?.b\n`c`", "Stmt((a?.b)`c`)"},
 		{"() => { const v=6; x={v} }", "Stmt(Params() => Stmt({ Decl(const Binding(v = 6)) Stmt(x={v}) }))"},
 		{`([]=l=>{let{e}={e}})`, `Stmt(([]=(Params(Binding(l)) => Stmt({ Decl(let Binding({ Binding(e) } = {e})) }))))`}, // go-fuzz
 	}
@@ -532,12 +535,12 @@ func TestParseError(t *testing.T) {
 		{"x=new.bad", "expected target instead of bad in new.target expression"},
 		{"x=import.bad", "expected meta instead of bad in import.meta expression"},
 		{"x=super", "expected [, (, or . instead of EOF in super expression"},
-		{"x=super(a", "expected ) instead of EOF in arguments"},
+		{"x=super(a", "expected , or ) instead of EOF in arguments"},
 		{"x=super[a", "expected ] instead of EOF in index expression"},
 		{"x=super.", "expected Identifier instead of EOF in dot expression"},
 		{"x=new super(b)", "expected [ or . instead of ( in super expression"},
 		{"x=import", "expected ( instead of EOF in import expression"},
-		{"x=import(5", "expected ) instead of EOF in arguments"},
+		{"x=import(5", "expected , or ) instead of EOF in arguments"},
 		{"x=new import(b)", "unexpected ( in expression"},
 		{"import", "expected String, Identifier, *, or { instead of EOF in import statement"},
 		{"import *", "expected as instead of EOF in import statement"},
@@ -562,6 +565,9 @@ func TestParseError(t *testing.T) {
 		{"export async", "expected function instead of EOF in export statement"},
 		{"throw", "unexpected EOF in expression"},
 		{"throw\n", "unexpected newline in throw statement"},
+		{"#private", "expected in instead of EOF in relational expression"},
+		{"new #private in z", "unexpected #private in expression"},
+		{"new\n#private in z", "unexpected #private in expression"},
 
 		// no declarations
 		{"if(a) function f(){}", "unexpected function in statement"},
